package de.laliluna.blob;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.SQLException;
import java.util.Random;

import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.dialect.PostgreSQLDialect;

import de.laliluna.hibernate.InitSessionFactory;

public class BlobTest {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		/** character LOB */
		// an oracle 9 workaround
		// createCharacterBlobWorkaround();
		createCharacterBlob();

		/** binary LOB */
		/*
		 * the following only works with annotation based mapping. You can
		 * enable/disable the mapping approach in the hibernate.cfg.xml
		 */
		// createBinaryBlobWithByteArray();
		/* works wit XML and annotation */
		createBinaryBlob();
	}

	/**
	 * This approach works with annotation and XML
	 */
	private static void createBinaryBlob() {
		byte byteArray[] = new byte[10000000];
		for (int i = 0; i < byteArray.length; i++) {
			byteArray[i] = '1';
		}
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		Image image = new Image();
		image.setImageAsBlob2(Hibernate.getLobCreator(session).createBlob(byteArray));
		session.save(image);
		session.getTransaction().commit();

		/* get one image from db and save it as file */
		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		Image imageReloaded = (Image) session.createQuery("from Image")
				.setMaxResults(1).uniqueResult();

		try {
			FileOutputStream outputStream = new FileOutputStream(new File(
					"image_file_blob_blob" + new Random().nextInt()));
			outputStream.write(imageReloaded.getImageAsBlob2().getBytes(1,
					(int) imageReloaded.getImageAsBlob2().length()));
			outputStream.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		session.getTransaction().commit();

		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		session.delete(image);
		session.getTransaction().commit();
	}

	/**
	 * Works only with annotation based mapping. Enable annotation based mapping
	 * in the hibernate.cfg.xml
	 */
	private static void createBinaryBlobWithByteArray() {
		byte byteArray[] = new byte[10000000];
		for (int i = 0; i < byteArray.length; i++) {
			byteArray[i] = '1';
		}
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		Image image = new Image();
		image.setImageAsBlob(byteArray);
		session.save(image);
		session.getTransaction().commit();

		/* get one image from db and save it as file */
		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		Image imageReloaded = (Image) session.createQuery("from Image")
				.setMaxResults(1).uniqueResult();

		try {
			FileOutputStream outputStream = new FileOutputStream(new File(
					"image_file_blob_array" + new Random().nextInt()));
			outputStream.write(imageReloaded.getImageAsBlob());
			outputStream.close();

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		session.getTransaction().commit();

		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		session.delete(image);
		session.getTransaction().commit();
	}

	private static void createCharacterBlob() {
		/* create a 10 MB String */
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < 1000000; i++)
			buffer.append("0123456789");

		Document document = new Document();
		document.setText(buffer.toString());
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		session.save(document);
		session.getTransaction().commit();

		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		Document documentReloaded = (Document) session.get(Document.class,
				document.getId());
		System.out.println("Length is: " + documentReloaded.getText().length());
		session.getTransaction().commit();

	}

	/**
	 * This workaround is suitable for Oracle 9 to create a clob larger than
	 * 4000 characters
	 */
	private static void createCharacterBlobWorkaround() {
		/* create a 10 MB String */
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < 1000000; i++)
			buffer.append("0123456789");

		Document document = new Document();

		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		/* initialize with short blob */
		document.setTextWorkaround(session.getLobHelper().createClob(" "));
		/* save before we continue */
		session.save(document);

		/* get a oracle clob to have access to outputstream */
		SerializableClob sc = (org.hibernate.lob.SerializableClob) document
				.getTextWorkaround();
		oracle.sql.CLOB clob = (oracle.sql.CLOB) sc.getWrappedClob();

		/* write the text to the clob outputstream */
		try {
			java.io.Writer pw = clob.getCharacterOutputStream();
			pw.write(buffer.toString());
			pw.close();
			session.getTransaction().commit();
		} catch (SQLException e) {
			throw new RuntimeException("Error saving the Lob", e);
		} catch (IOException e) {
			throw new RuntimeException("Error saving the Lob", e);
		}
		session.getTransaction().commit();

		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		Document documentReloaded = (Document) session.get(Document.class,
				document.getId());
		StringBuffer textFromWorkaround = new StringBuffer();
		try {
			BufferedReader bufferedClobReader = new BufferedReader(
					documentReloaded.getTextWorkaround().getCharacterStream());

			String line = null;
			while ((line = bufferedClobReader.readLine()) != null) {
				textFromWorkaround.append(line);
			}
			bufferedClobReader.close();
		} catch (IOException e) {
			throw new RuntimeException("Error reading the Lob", e);
		} catch (SQLException e) {
			throw new RuntimeException("Error reading the Lob", e);
		}
		session.getTransaction().commit();

		System.out.println("length of text: " + textFromWorkaround.length());

	}
}
