package de.laliluna.example.jsf;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIData;
import javax.faces.context.FacesContext;


import org.hibernate.StaleObjectStateException;

import de.laliluna.example.domain.Hedgehog;
import de.laliluna.example.service.HedgehogService;
import de.laliluna.example.service.HedgehogServiceImp;

public class HedgehogBean {
	private static org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(HedgehogBean.class);

	private HedgehogService hedgehogService = new HedgehogServiceImp();

	private Hedgehog currentHedgehog = new Hedgehog();

	private UIData hedgehogTable;

	private String currentSortColumn = "";

	private boolean sortAsc = true;

	public HedgehogBean() {

	}

	public void sort(String sortColumn) {

		if (sortColumn == null)
			throw new IllegalArgumentException(
					"Argument sortColumn must not be null.");
		// inverse order
		if (currentSortColumn.equals(sortColumn))
			sortAsc = !sortAsc;
		else
			currentSortColumn = sortColumn;

	}

	public String selectHedgehog() {

		currentHedgehog = (Hedgehog) hedgehogTable.getRowData();
		log.debug("Selected: " + currentHedgehog);

		if (currentHedgehog != null)
			return Globals.SUCCESS;
		else
			return Globals.FAILURE;
	}

	public String save() {

		log.debug("saving: " + currentHedgehog);
		try {
			hedgehogService.saveOrUpdateHedgehog(currentHedgehog);
			currentHedgehog = null;
		} catch (StaleObjectStateException e) {
			currentHedgehog = hedgehogService.findById(currentHedgehog.getId());
			FacesContext.getCurrentInstance().addMessage(
					null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, Globals
							.getText("errors_rowWasUpdatedByOtherUser", null),
							""));
			return Globals.FAILURE;
		}

		return Globals.SUCCESS;

	}

	/**
     * returns a sorted list of hedgehogs
     * 
     * @return
     */
	public List<Hedgehog> getAll() {
		
		List<Hedgehog> list = hedgehogService.findAll();

		if (currentSortColumn != null && currentSortColumn.equals("name")) {
			final int inverse = (sortAsc ? 1 : -1);
			Collections.sort(list, new Comparator<Hedgehog>() {

				public int compare(Hedgehog o1, Hedgehog o2) {
					if (o1 == null || o2.getName() == null)
						return (-1*inverse);
					if (o2 == null || o2.getName() == null)
						return 1*inverse;
					return inverse * o1.getName().compareTo(o2.getName());
				}

			});
		}
		return list;
	}

	/**
     * @return the hedgehogTable
     */
	public UIData getHedgehogTable() {
		return hedgehogTable;
	}

	/**
     * @param hedgehogTable
     *            the hedgehogTable to set
     */
	public void setHedgehogTable(UIData hedgehogTable) {
		this.hedgehogTable = hedgehogTable;
	}

	/**
     * @return the currentHedgehog
     */
	public Hedgehog getCurrentHedgehog() {
		return currentHedgehog;
	}

	/**
     * @param currentHedgehog
     *            the currentHedgehog to set
     */
	public void setCurrentHedgehog(Hedgehog currentHedgehog) {
		this.currentHedgehog = currentHedgehog;
	}

	/**
     * @return the currentSortcolumn
     */
	public String getCurrentSortColumn() {
		return currentSortColumn;
	}

	/**
     * @param currentSortcolumn
     *            the currentSortcolumn to set
     */
	public void setCurrentSortColumn(String currentSortcolumn) {
		sort(currentSortcolumn);
	}

	/**
     * @return the sortAsc
     */
	public boolean isSortAsc() {
		return sortAsc;
	}

	/**
     * @param sortAsc
     *            the sortAsc to set
     */
	public void setSortAsc(boolean sortAsc) {
		this.sortAsc = sortAsc;
	}

}
