package de.laliluna.example.service;

import java.util.List;

import de.laliluna.example.domain.Hedgehog;
import de.laliluna.example.domain.WinterAddress;

public interface HedgehogService {

	public enum SortBy{NAME_DESC, NAME_ASC, ID_ASC, ID_DESC}
	/**
	 * creates a new hedgehog
	 * @param hedgehog
	 */
	public void saveOrUpdateHedgehog(Hedgehog hedgehog);

	/**
	 * find all hedgehogs and returns them.
	 * @return
	 */
	public List<Hedgehog> findAll();
	
	/**
	 * returns all hedgehogs between the firstItem and the lastItem sorted by the sortBy parameter.
	 *   
	 * @param firstItem
	 * @param lastItem
	 * @param sortBy
	 * @return
	 */
	public List<Hedgehog> findSubset(int firstItem, int lastItem, SortBy sortBy);
	
	/**
	 * find a hedgehog by primary key
	 * @param id
	 * @return the hedgehog or null if not found
	 */
	public Hedgehog findById(Integer id);

	/**
	 * updates a hedgehog
	 * @param hedgehog
	 */
	public void update(Hedgehog hedgehog);
	
	/**
	 * hibernates a hedgehog at a specific address
	 * @param hedgehog
	 * @param winterAddress
	 */
	public void hibernate(Hedgehog hedgehog, WinterAddress winterAddress);

	/**
	 * counts all hedgehogs in database
	 * @return
	 */
	public int countAllHedgehogs();

}