package de.laliluna.example.service;

import java.text.MessageFormat;
import java.util.List;


import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.StaleObjectStateException;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;

import de.laliluna.example.InitSessionFactory;
import de.laliluna.example.domain.DaoFactory;
import de.laliluna.example.domain.Hedgehog;
import de.laliluna.example.domain.HedgehogDao;
import de.laliluna.example.domain.WinterAddress;

public class HedgehogServiceImp implements HedgehogService {

	private HedgehogDao hedgehogDao;

	private static org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(HedgehogServiceImp.class);

	public HedgehogServiceImp() {
		hedgehogDao = DaoFactory.createHedgehogDao();
	}

	public Session getSession() {
		return InitSessionFactory.getInstance().getCurrentSession();
	}

	/*
     * (non-Javadoc)
     * 
     * @see de.laliluna.example.service.HedgehogService#createNewHedgehog(de.laliluna.example.domain.Hedgehog)
     */

	public void saveOrUpdateHedgehog(Hedgehog hedgehog) {
		try {
			getSession().beginTransaction();
			hedgehogDao.saveOrUpdate(hedgehog);
			getSession().getTransaction().commit();
		} catch (StaleObjectStateException e) {
			try {
				getSession().getTransaction().rollback();
				log.debug("Rolled back transaction");
			} catch (RuntimeException e1) {
				log.error("Error rolling back transaction");
			}
			throw e;
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see de.laliluna.example.service.HedgehogService#findAll()
     */

	public List<Hedgehog> findAll() {
		getSession().beginTransaction();
		List<Hedgehog> list = hedgehogDao.findAll();
		getSession().getTransaction().commit();
		return list;

	}

	/*
     * (non-Javadoc)
     * 
     * @see de.laliluna.example.service.HedgehogService#findById(Integer id)
     */

	public Hedgehog findById(Integer id) {
		getSession().beginTransaction();
		Hedgehog hedgehog = hedgehogDao.findById(id);
		getSession().getTransaction().commit();
		return hedgehog;
	}

	/*
     * (non-Javadoc)
     * 
     * @see de.laliluna.example.service.HedgehogService#update(Hedgehog
     *      hedgehog)
     */

	public void update(Hedgehog hedgehog) {
		try {
			getSession().beginTransaction();

			hedgehogDao.update(hedgehog);
			getSession().getTransaction().commit();
		} catch (StaleObjectStateException e) {
			try {
				getSession().getTransaction().rollback();
			} catch (HibernateException e1) {
				log.error("Error rolling back transaction");
			}
			throw e;
		}

	}

	/*
     * (non-Javadoc)
     * 
     * @see de.laliluna.example.service.HedgehogService#findSubset(int, int,
     *      de.laliluna.example.service.HedgehogService.SortBy)
     */
	public List<Hedgehog> findSubset(int firstItem, int lastItem, SortBy sortBy) {
		// set default order
		Order order = Order.asc("id");

		switch (sortBy) {
		case ID_ASC:
			order = Order.asc("id");
			break;
		case ID_DESC:
			order = Order.desc("id");
			break;
		case NAME_ASC:
			order = Order.asc("name");
			break;
		case NAME_DESC:
			order = Order.desc("name");
			break;
		}
		log.debug(MessageFormat.format("Get Item: {0} to {1} sort by {2}",
				new Object[] { firstItem, lastItem, sortBy }));
		getSession().beginTransaction();
		List<Hedgehog> list = hedgehogDao.findSubSetByCriteria(firstItem,
				lastItem, order);
		getSession().getTransaction().commit();
		return list;
	}

	/*
     * (non-Javadoc)
     * 
     * @see de.laliluna.example.service.HedgehogService#hibernate(de.laliluna.example.domain.Hedgehog,
     *      de.laliluna.example.domain.WinterAddress)
     */

	public void hibernate(Hedgehog hedgehog, WinterAddress winterAddress) {
		getSession().beginTransaction();
		hedgehog.getAddresses().add(winterAddress);
		winterAddress.setHedgehog(hedgehog);
		hedgehogDao.update(hedgehog);
		getSession().getTransaction().commit();
	}

	public void setHedgehogDao(HedgehogDao hedgehogDao) {
		this.hedgehogDao = hedgehogDao;
	}

	/*
     * (non-Javadoc)
     * 
     * @see de.laliluna.example.service.HedgehogService#countAllHedgehogs()
     */
	public int countAllHedgehogs() {
		int result = 0;
		getSession().beginTransaction();
		result = ((Integer) getSession().createCriteria(Hedgehog.class)
				.setProjection(Projections.rowCount()).uniqueResult())
				.intValue();
		getSession().getTransaction().commit();
		return result;
	}

	public List<Hedgehog> getAll() {
		return findAll();
	}

}
