
package de.laliluna.relation.typed;

import java.util.Iterator;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Session;

import de.laliluna.example.BasicTest;
import de.laliluna.hibernate.InitSessionFactory;


public class TestRelationTyped extends BasicTest {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		clean();
		create();

		query();

	}

	private static void clean() {
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		
		session.beginTransaction();

		session.createQuery("delete from BillingReaderAddress").executeUpdate();
		session.createQuery("delete from DeliveryReaderAddress").executeUpdate();
		session.createQuery("delete from TutorialReader").executeUpdate();
		
		session.getTransaction().commit();
		
	}

	private static void query() {
		log.debug("query");
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		List list = null;
		session.beginTransaction();

		log.debug("select all billing addresses");
		list = session.createQuery("from BillingReaderAddress").list();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			ReaderAddress element = (ReaderAddress) iter.next();
			log.debug("{}", element);
		}

		log.debug("select tutorial reader with billing address in Frankfurt");
		list = session.createQuery("from TutorialReader r where r.billingAddress.city='Frankfurt'")
				.list();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			TutorialReader reader = (TutorialReader) iter.next();
			log.debug("{}", reader);
			log.debug("{}", reader.getBillingAddress());
			log.debug("{}", reader.getDeliveryAddress());
		}
		log.debug("select all tutorial reader ");
		list = session.createQuery("from TutorialReader")
				.list();
		for (Iterator iter = list.iterator(); iter.hasNext();) {
			TutorialReader reader = (TutorialReader) iter.next();
			log.debug("{}", reader);
			log.debug("{}", reader.getBillingAddress());
			log.debug("{}", reader.getDeliveryAddress());
		}
		session.getTransaction().commit();

	}

	private static void create() {
		log.debug("create");
		Session session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		TutorialReader reader = new TutorialReader();
		reader.setName("Sebastian");
		session.save(reader);
		ReaderAddress billing = new ReaderAddress(new ReaderAddressId(reader
				.getId(), ReaderAddressId.BILLING), "Alte Landstrasse",
				"Frankfurt");
		ReaderAddress delivery = new ReaderAddress(new ReaderAddressId(reader
				.getId(), ReaderAddressId.DELIVERY), "Neue Landstrasse",
				"Frankfurt");
		reader.setBillingAddress(billing);
		reader.setDeliveryAddress(delivery);
		session.getTransaction().commit();
		
		session = InitSessionFactory.getInstance().getCurrentSession();
		session.beginTransaction();
		
		// show that not allowed types do not work
		try {
			ReaderAddress badAddress = new ReaderAddress(new ReaderAddressId(reader
					.getId(), "bad address type"), "Alte Landstrasse",
					"Frankfurt");
			session.save("BillingReaderAddress", badAddress);
			session.getTransaction().commit();
		} catch (HibernateException e) {
			e.printStackTrace();
			try {
				session.getTransaction().rollback();
			} catch (HibernateException e1) {
			
			}
		}
		
		
	}

}
