
package de.laliluna.hibernate;

import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.service.ServiceRegistryBuilder;

import java.util.Properties;

// import org.hibernate.cfg.AnnotationConfiguration;

/**
 * @author hennebrueder This class garanties that only one single SessionFactory
 *         is instanciated and that the configuration is done thread safe as
 *         singleton. Actually it only wraps the Hibernate SessionFactory. When
 *         a JNDI name is configured the session is bound to to JNDI, else it is
 *         only saved locally. You are free to use any kind of JTA or Thread
 *         transactionFactories.
 */
public class InitSessionFactory {

    private static org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(InitSessionFactory.class);

    /**
     * The single instance of hibernate SessionFactory
     */
    private static org.hibernate.SessionFactory sessionFactory;

    /**
     * Default constructor. It is private to guaranty singleton
     */
    private InitSessionFactory() {
    }


    /**
     * Returns the single instance of the session factory
     *
     * @return
     */
    public static SessionFactory getInstance() {
        return sessionFactory;
    }

    public static void initSessionFactory(CacheSzenario cacheSzenario) {
        Properties properties = null;
        final Configuration cfg = new Configuration();
        cfg.configure();

        switch (cacheSzenario) {
            case NOCACHE:
                cfg.addFile("src/main/resources/nocache.hbm.xml");
                break;
            case READCACHE:
                cfg.addFile("src/main/resources/readcache.hbm.xml");
                break;
            case READWRITECACHE:
                cfg.addResource("readwritecache.hbm.xml");
                break;
            case NONRESTRICTEDCACHE:
                cfg.addFile("src/main/resources/nonrestricted.hbm.xml");
                break;
            case TRANSACTIONALCACHE:
                cfg.addFile("src/main/resources/transactional.hbm.xml");
                break;

            case NONRESTRICTEDQUERYDCACHE:
                cfg.addFile("src/main/resources/nonrestricted.hbm.xml");
                properties = new Properties();
                properties.put("hibernate.cache.use_query_cache", "true");

                cfg.addProperties(properties);
                break;

            case QUERYCACHE:
                cfg.addFile("src/main/resources/readwritecache.hbm.xml");
                properties = new Properties();
                properties.put("hibernate.cache.use_query_cache", "true");

                cfg.addProperties(properties);
                break;

            default:
                throw new RuntimeException(
                        "specified CacheSzenario is not supported");
        }
        final ServiceRegistry serviceRegistry = new ServiceRegistryBuilder()
                .applySettings(cfg.getProperties())
                .buildServiceRegistry();
        sessionFactory = cfg.buildSessionFactory(serviceRegistry);

    }

}
